/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.collections.testlist;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import org.builtu.exceptions.FatalError;


public abstract class AbstractTestFixedList<E> extends AbstractTestList<E> {
  protected ListOperation<E> makeModifier(final List<E> ref) 
  {
    final int idx = -2 + rnd(ref.size()+3);
    final E e = makeElement();
    final E existingElement = ref.size()==0 ? 
          makeElement() : ref.get(rnd(ref.size()));
    final List<E> coll         = randomColl(ref),
                  existingColl = randomCollectionOfExistingElements(ref);
    final List<E> clone = cloneList(ref);
    final int skip = rnd(ref.size());
    
    
    final int opCode = rnd(30);
    switch(opCode) {
    case 0:
    case 1: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return list.get(idx);
      }}; 
    case 2: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return list.set(idx, e);
      }}; 
    case 3: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return b2e(list.contains(e), e);
      }}; 
    case 4: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        final E e = list.get(idx);
        return b2e(list.contains(e), e);
      }}; 
    case 5: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return b2e(list.contains(existingElement), existingElement);
      }}; 
    case 6: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return b2e(list.containsAll(coll), e);
      }}; 
    case 7: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return b2e(list.containsAll(existingColl), e);
      }}; 
    case 8: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return b2e(list.equals(list), e);
      }}; 
    case 9: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return b2e(list.equals(clone), e);
      }}; 
    case 10: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return b2e(list.indexOf(e)>=0, e);
      }}; 
    case 11: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return b2e(e.equals(list.get(list.indexOf(e))), e);
      }}; 
    case 12: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return b2e(list.indexOf(existingElement)>=0, existingElement);
      }}; 
    case 13: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return b2e(e.equals(list.get(list.indexOf(existingElement))), 
            existingElement);
      }}; 
    case 14: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return b2e(list.isEmpty(), e);
      }}; 
    case 15: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return b2e(list.iterator().hasNext(), e);
      }}; 
    case 16: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return list.iterator().next();
      }}; 
    case 17: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        final Iterator<E> it = list.iterator();
        it.next();
        return list.iterator().next();
      }}; 
    case 18: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        final Iterator<E> it = list.iterator();
        it.next();
        it.next();
        return it.next();
      }}; 
    case 19: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        for(int i=0; i<skip; i++) list.iterator().next();
        return list.iterator().next();
      }}; 
    case 20: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        final int idx = list.lastIndexOf(e); 
        return b2e(idx>=0, e);
      }}; 
    case 21: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return b2e(list.lastIndexOf(existingElement)>=0, existingElement);      
      }};
    case 22: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return list.listIterator().next();      
      }};
    case 23: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        final ListIterator<E> it = list.listIterator();
        it.next();
        return it.previous();      
      }};
    case 24: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        final ListIterator<E> it = list.listIterator();
        for(int i=0; i<skip; i++) it.next();
        return it.next();    
      }};
    case 25: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        final ListIterator<E> it = list.listIterator();
        for(int i=0; i<skip; i++) it.next();
        return it.previous();    
      }};
    case 26: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return list.subList(idx, idx+skip).get(0);
      }};
    case 27: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return list.subList(idx, idx+skip).get(1);
      }};
    case 28: return new ListOperation<E>(){
      @Override public E modifyList(final List<E> list)
      {
        return list.subList(idx, idx+skip).get(skip);
      }};
    case 29: return new ListOperation<E>(){
      @SuppressWarnings("unchecked") @Override 
      public E modifyList(final List<E> list)
      {
        return (E) list.toArray()[idx];
      }};
      
    default: fail("Bad opcode "+opCode);
    }
    throw new FatalError("Can't reach here!!!");
  }
  protected List<E> cloneList(final List<E> ref)
  {
    final List<E> res = new ArrayList<E>();
    for (final E e : ref) res.add(e);
    return res;
  }
  protected List<E> randomCollectionOfExistingElements(final List<E> ref)
  {
    final List<E> res = new ArrayList<E>();
    for (int i=0; i<rnd(ref.size()); i++) {
      final int idx = rnd(ref.size());
      res.add(ref.get(idx));
    }
    return res;
  }
  protected List<E> randomColl(final List<E> ref)
  {
    final List<E> res = randomCollectionOfExistingElements(ref);
    for (int i=0; i<10; i++) {
      res.add(makeElement());
    }
    Collections.shuffle(res);
    return res;
  }
  protected static<E> E b2e(final boolean b, final E e) {
    return b ? e : null;
  }
}
