/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.services.resources;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import org.apache.log4j.Logger;
import org.builtu.exceptions.FatalError;


public class ClasspathResourceLocation implements ResourceLocation {
	
	private static final Logger log = Logger.getLogger(ClasspathResourceLocation.class);
	
	private final String root;
	
	public ClasspathResourceLocation(final String root) {
		this.root = root;
	}

	@Override public Resource resolve(final String path) {
		final URL url = ResourceRoot.class.getResource(root+path);
		if (url == null) return null;
		if (url.getProtocol().equals("file")) {
			final File file = new File(url.getPath());
			if (!file.isFile()) {
				log.info("File not found: "+file);
				return null;
			}
			log.info("File found: "+file);
			return new FileResource(file);
		}
		final Resource res; 
		try {
			res = new PrefetchedResource(url);
		} catch (final IOException e) {
			throw new FatalError(e);
		}
		log.info("Resource found: "+url);
		return res;
	}

}
