/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.services.resources;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.log4j.Logger;
import org.builtu.exceptions.FatalError;


public class UrlResourceLocation implements ResourceLocation {
	private final String root;
	
	private static final Logger log = Logger.getLogger(UrlResourceLocation.class);
	
	public UrlResourceLocation(final String root) {
		assert validUrl(root);
		this.root = root;
	}

	private static boolean validUrl(final String root) {
		try {
			new URL(root);
			return true;
		} catch (final MalformedURLException e) {
			throw new FatalError(e);
		}
	}

	@Override public Resource resolve(final String path) {
		final URL url;
		try {
			url = new URL(root+path);
		} catch (final MalformedURLException e) {
			throw new FatalError(e);
		} 
		
		final Resource res; 
		try {
			res = new PrefetchedResource(url);
		} catch (final IOException e) {
			log.debug("Resource not found: "+url, e);
			return null;
		}
		log.info("Resource found: "+url);
		return res;
	}
}
