/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.services.resources;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import junit.framework.Assert;

import org.builtu.services.resources.Resource;
import org.builtu.services.resources.ResourceRoot;
import org.builtu.testing.BasicTestCase;
import org.builtu.util.commutil.IOUtil;
import org.builtu.util.commutil.StringUtil;
import org.junit.Before;
import org.junit.Test;


public class TestResourceRoot extends BasicTestCase {
	
	private ResourceRoot root;
	
	@Before public void create() {
		root = new ResourceRoot();
	}
	
	@Test(expected=IllegalArgumentException.class) public void test_get_bad_path() throws IOException {
		root.get("path has to start with slash");
	}
	
	@Test public void test_test_jpg() throws IOException {
		final byte[] buffer = new byte[1024];
		int count = 0;
		final Resource r = root.get("/img/background/test.jpg");
		Assert.assertEquals(17493, r.size());
		final InputStream in = r.openStream();
		try {
			int read = in.read(buffer);
			while(read>=0) {
				count += read;
				read = in.read(buffer);
			}
		} finally {
			in.close();
		}
		Assert.assertEquals(17493, count);
	}
	
	@Test public void test_temp_file() throws IOException {
		final File temp = super.createTempFile();
		IOUtil.writeASCIIFileContents(temp, "Hello");
		root = new ResourceRoot("file://"+temp.getParent());
		final Resource res = root.get('/'+temp.getName());
		final InputStream in = res.openStream();
		try {
			Assert.assertEquals("Hello", IOUtil.readContents(in, StringUtil.asciiCharset));
		} finally {
			in.close();
		}
	}
	
	@Test public void testDoesNotServeFolders() {
		Assert.assertSame(null, root.get("/img/"));
	}
	
	@Test public void testDoesNotServeJarFolders() {
		Assert.assertSame(null, root.get("/builtu/config"));
	}
}
