/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.commutil;

import java.util.HashMap;
import java.util.Map;

import junit.framework.Assert;

import org.builtu.util.commutil.Util;
import org.junit.Before;
import org.junit.Test;


public class TestUtilGet4 {
	private Map<String, Map<Byte, Map<Integer, Map<Long, Map<Float, Double>>>>> m5;
	private Map<Byte, Map<Integer, Map<Long, Map<Float, Double>>>> m4;
	private Map<Integer, Map<Long, Map<Float, Double>>> m3;
	private Map<Long, Map<Float, Double>> m2;
	private Map<Float, Double> m1;
	
	@Before public void fillMap() {
		m1 = new HashMap<Float, Double>(1);
		m1.put(1.0f, 1.0);
		m2 = new HashMap<Long, Map<Float, Double>>(1);
		m2.put(1L, m1);
		m3 = new HashMap<Integer, Map<Long, Map<Float, Double>>>(1);
		m3.put(1, m2);
		m4 = new HashMap<Byte, Map<Integer, Map<Long, Map<Float, Double>>>>(1);
		m4.put((byte)1, m3);
		m5 = new HashMap<String, Map<Byte, Map<Integer, Map<Long, Map<Float, Double>>>>>(1);
		m5.put("1", m4);
	}
	

	@Test public void test1() {
		Assert.assertTrue(1.0 == Util.get(m1, 1.0f));
		Assert.assertSame(null, Util.get(m1, 2.0f));
		Assert.assertSame(m1, Util.get(m2, 1L));
		Assert.assertSame(null, Util.get(m2, 2L));
		Assert.assertSame(m2, Util.get(m3, 1));
		Assert.assertSame(null, Util.get(m3, 2));
		Assert.assertSame(m3, Util.get(m4, (byte)1));
		Assert.assertSame(null, Util.get(m4, (byte)2));
		Assert.assertSame(m4, Util.get(m5, "1"));
		Assert.assertSame(null, Util.get(m5, "2"));
	}
	
	@Test public void test2() {
		Assert.assertTrue(1.0 == Util.get(m2, 1L, 1.0f));
		Assert.assertSame(null, Util.get(m2, 1L, 2.0f));
		Assert.assertSame(null, Util.get(m2, 2L, 1.0f));
		Assert.assertSame(m1, Util.get(m3, 1, 1L));
		Assert.assertSame(null, Util.get(m3, 1, 2L));
		Assert.assertSame(null, Util.get(m3, 2, 1L));
		Assert.assertSame(m2, Util.get(m4, (byte)1, 1));
		Assert.assertSame(null, Util.get(m4, (byte)1, 2));
		Assert.assertSame(null, Util.get(m4, (byte)2, 1));
		Assert.assertSame(m3, Util.get(m5, "1", (byte)1));
		Assert.assertSame(null, Util.get(m5, "1", (byte)2));
		Assert.assertSame(null, Util.get(m5, "2", (byte)1));
	}
	
	@Test public void test3() {
		Assert.assertTrue(1.0 == Util.get(m3, 1, 1L, 1.0f));
		Assert.assertSame(null, Util.get(m3, 1, 1L, 2.0f));
		Assert.assertSame(null, Util.get(m3, 1, 2L, 1.0f));
		Assert.assertSame(null, Util.get(m3, 3, 1L, 1.0f));
		Assert.assertSame(m1, Util.get(m4, (byte)1, 1, 1L));
		Assert.assertSame(null, Util.get(m4, (byte)1, 1, 2L));
		Assert.assertSame(null, Util.get(m4, (byte)1, 2, 1L));
		Assert.assertSame(null, Util.get(m4, (byte)2, 1, 1L));
		Assert.assertSame(m2, Util.get(m5, "1", (byte)1, 1));
		Assert.assertSame(null, Util.get(m5, "1", (byte)1, 2));
		Assert.assertSame(null, Util.get(m5, "1", (byte)2, 1));
		Assert.assertSame(null, Util.get(m5, "2", (byte)1, 1));
	}
	
	@Test public void test4() {
		Assert.assertTrue(1.0 == Util.get(m4, (byte)1, 1, 1L, 1.0f));
		Assert.assertSame(m1, Util.get(m5, "1", (byte)1, 1, 1L));
		Assert.assertSame(null, Util.get(m5, "1", (byte)1, 1, 2L));
		Assert.assertSame(null, Util.get(m5, "1", (byte)1, 2, 1L));
		Assert.assertSame(null, Util.get(m5, "1", (byte)2, 1, 1L));
		Assert.assertSame(null, Util.get(m5, "2", (byte)1, 1, 1L));
	}
}
