package org.builtu.webutil.ordreq;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.ReentrantLock;

public class OrdreqLock {
	
	private final ConcurrentHashMap<String, ReentrantLock> locks = 
			new ConcurrentHashMap<String, ReentrantLock>();

	private final Object compactLock = new Object();
	private boolean compactAllowed=true;
	
	public boolean wouldBlock(final String id) {
		synchronized(compactLock) {
			compactAllowed = false;
		}
		
		ReentrantLock l = locks.get(id);
		if (l == null) {
			locks.putIfAbsent(id, new ReentrantLock());
			l = locks.get(id);
		}
		
		final boolean res = l.isLocked();
		
		synchronized(compactLock) {
			compactAllowed = true;
		}
		
		return res;
	}
	
	public void lockInterruptibly(final String id) throws InterruptedException {
		synchronized(compactLock) {
			compactAllowed = false;
		}
		
		ReentrantLock l = locks.get(id);
		if (l == null) {
			locks.putIfAbsent(id, new ReentrantLock());
			l = locks.get(id);
		}
		
		l.lockInterruptibly();
		
		synchronized(compactLock) {
			compactAllowed = true;
		}
	}
	
	public void lock(final String id) {
		synchronized(compactLock) {
			compactAllowed = false;
		}
		
		ReentrantLock l = locks.get(id);
		if (l == null) {
			locks.putIfAbsent(id, new ReentrantLock());
			l = locks.get(id);
		}
		
		l.lock();
		
		synchronized(compactLock) {
			compactAllowed = true;
		}
	}
	
	
	public synchronized void unlock(final String id) {
		final ReentrantLock l = locks.get(id);
		if (l == null) {
			throw new IllegalMonitorStateException("No lock for "+id);
		}
		if (!l.isHeldByCurrentThread()) {
			throw new IllegalMonitorStateException("Lock for "+id+" is held by another thread");
		}
		
		if (!l.hasQueuedThreads() && l.getHoldCount() == 1) {
			boolean compacted = false;
			while (!compacted) {
				synchronized(compactLock) {
					if (compactAllowed) {
						if (!l.hasQueuedThreads() && l.getHoldCount() == 1)	locks.remove(id);		
						compacted = true;
					}
				}
			}
		}
		l.unlock();
	}
	
	
	
}
